//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: ContextItemQueryCommand.cpp,v 1.5 2004/10/18 22:52:13 gmf Exp $
//

#include "ContextItemQueryCommand.hpp"

using namespace DbXml;
using namespace std;

string ContextItemQueryCommand::getCommandName() const
{
	return "ciquery";
}

string ContextItemQueryCommand::getBriefHelp() const
{
	return "Executes an expression using the last results as the context item for it";
}

string ContextItemQueryCommand::getMoreHelp() const
{
	return
		string("Takes one argument - the query to be run. This is run using the\n") +
		string("last returned results as context items for the query (meaning they\n") +
		string("can be accessed with \".\"). It uses XmlManager::prepare() to\n") +
		string("create an XmlQueryExpression, then uses the execute method on\n") +
		string("that, passing in the last results as an argument.");
}

void ContextItemQueryCommand::execute(Args &args, Environment &env)
{
	if(args.size() != 2) {
		throw CommandException("Wrong number of arguments");
	}
	env.testResults();
	XmlResults newResults = env.db().createResults();
	env.results()->reset();

	if(env.txn()) {
		XmlQueryExpression expr = env.db().prepare(*env.txn(), args[1], env.context());

		XmlValue value;
		while(env.results()->next(value)) {
			XmlResults tmpResults = expr.execute(*env.txn(), value, env.context());

			XmlValue tmpValue;
			while(tmpResults.next(tmpValue)) {
				newResults.add(tmpValue);
			}
		}
	}
	else {
		XmlQueryExpression expr = env.db().prepare(args[1], env.context());

		XmlValue value;
		while(env.results()->next(value)) {
			XmlResults tmpResults = expr.execute(value, env.context());

			XmlValue tmpValue;
			while(tmpResults.next(tmpValue)) {
				newResults.add(tmpValue);
			}
		}
	}

	env.deleteResults();
	env.results() = new XmlResults(newResults);

	if(env.verbose()) cout << env.results()->size() << " objects returned for expression '"
			       << args[1] << "'\n" << endl;
}

