//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: DeleteIndexCommand.cpp,v 1.4 2004/11/08 16:08:08 jsnelson Exp $
//

#include "DeleteIndexCommand.hpp"

using namespace DbXml;
using namespace std;

string DeleteIndexCommand::getCommandName() const
{
  return "delindex";
}

string DeleteIndexCommand::getBriefHelp() const
{
  return "Delete an index from the default container";
}

string DeleteIndexCommand::getMoreHelp() const
{
  return
    string("Takes optional arguments of the namespace and name of the element\n") +
    string("or attribute, and the index descriptor string. If the namespace and\n") +
    string("name are not set, then this command deletes from the default index.\n") +
    string("This command uses the XmlContainer::deleteIndex() and\n") +
    string("XmlContainer::deleteDefaultIndex() methods.\n");
}

void DeleteIndexCommand::execute(Args &args, Environment &env)
{
  if(args.size() != 2 && args.size() != 4) {
    throw CommandException("Wrong number of arguments");
  }
  env.testContainer();

  if(env.verbose()) {
	  if(args.size() == 2) cout << "Deleting default index type: " << args[1] << endl;
	  else cout << "Deleting index type: " << args[3] << " from node: {" << args[1] << "}:" << args[2] << endl;
  }

  if(env.txn()) {
	  XmlTransaction myTxn = env.childTransaction();
	  if(args.size() == 2) env.container()->deleteDefaultIndex(myTxn, args[1], env.uc());
	  else env.container()->deleteIndex(myTxn, args[1], args[2], args[3], env.uc());
	  myTxn.commit();
  }
  else {
	  if(args.size() == 2) env.container()->deleteDefaultIndex(args[1], env.uc());
	  else env.container()->deleteIndex(args[1], args[2], args[3], env.uc());
  }
}

