/*
 * Copyright (c) 2002, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef __PROXYMEMORYMANAGER_HPP
#define __PROXYMEMORYMANAGER_HPP

#include <pathan/Pathan.hpp>
#include <pathan/XPath2MemoryManager.hpp>

#include <xercesc/util/RefHashTableOf.hpp>

XERCES_CPP_NAMESPACE_BEGIN
class DOMNode;
class XMLGrammarPool;
XERCES_CPP_NAMESPACE_END

class Sequence;
class VariableStore;
class FunctionLookup;
class DynamicContext;
class Collation;
class CollationHelper;
class PathanNSResolver;
class Item;
class Node;
class StringPool;

class PATHAN_EXPORT ProxyMemoryManager : public XPath2MemoryManager
{
public:
  ProxyMemoryManager(XERCES_CPP_NAMESPACE_QUALIFIER MemoryManager *parent);
  virtual ~ProxyMemoryManager();

  // from MemoryManager
  virtual void* allocate(size_t numElements);
  virtual void deallocate(void* p);
  
  /** Returns a copy of the given string */
  virtual const XMLCh* getPooledString(const XMLCh *src);
  /** Returns a copy of the transcoding of the given string */
  virtual const XMLCh* getPooledString(const char *src);

  /** Use with extreme caution! */
  virtual void reset();

  virtual void dumpStatistics();
  
  /** create a collation */
  virtual Collation* createCollation(CollationHelper* helper);

  /** create a resolver */
  virtual PathanNSResolver* createNSResolver(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *resolverNode);
  
  /** create a store for variables */
  virtual VariableStore* createVariableStore();

  /** create a store for variables */
  virtual VariableTypeStore* createVariableTypeStore();

  /** create a function table */
  virtual FunctionLookup* createFunctionTable();

  /** create a ATDecimalOrDerived for the given integer */
  virtual ATDecimalOrDerived* createInteger(int value);

protected:
  void initialise();
  void releaseAll();

private:
  XERCES_CPP_NAMESPACE_QUALIFIER MemoryManager *parent_;

  struct PATHAN_EXPORT MemList {
    MemList *prev;
    MemList *next;
  };
  MemList *fCurrentBlock;

  StringPool *fStringPool;

  XERCES_CPP_NAMESPACE_QUALIFIER RefHashTableOf<ATDecimalOrDerived>* fIntegerPool;
};

#endif //__XPATH2MEMORYMANAGERIMPL_HPP

